<?php

namespace Drupal\couchbasedrupal;

use Drupal\Core\DependencyInjection\ServiceModifierInterface;
use Drupal\Core\DependencyInjection\ServiceProviderInterface;
use Drupal\Core\DependencyInjection\ContainerBuilder;

use Drupal\Core\Site\Settings;

use Drupal\couchbasedrupal\Cache\CouchbaseBackendFactory;
use Drupal\couchbasedrupal\Cache\CouchbaseTagsChecksum;
use Drupal\couchbasedrupal\Lock\CouchbaseLockBackend;
use Drupal\Core\Lock\DatabaseLockBackend;
use Drupal\Core\Lock\PersistentDatabaseLockBackend;
use Drupal\Core\Cache\DatabaseCacheTagsChecksum;

class CouchbasedrupalServiceProvider implements ServiceProviderInterface {
  /**
   * {@inheritdoc}
   */
  public function register(ContainerBuilder $container) {
    // Set chained fast backend persitent storage
    if ($container->has('cache.backend.chainedfast')) {
      $definition  = $container->getDefinition('cache.backend.chainedfast');
      $args = $definition->getArguments();
      // Add the missins optional arguments
      if (!isset($args[1])) {
        $args[] = 'cache.backend.couchbase';
        $definition->setArguments($args);
      }
    }

    // Set chained fast backend persitent storage
    if ($container->has('cache.backend.superchainedfast')) {
      $definition  = $container->getDefinition('cache.backend.superchainedfast');
      $args = $definition->getArguments();
      // Add the missins optional arguments
      if (!isset($args[1])) {
        $args[] = 'cache.backend.couchbase';
        $definition->setArguments($args);
      }
    }

    // Set chained fast backend persitent storage
    if ($container->has('cache.rawbackend.chainedfast')) {
      $definition  = $container->getDefinition('cache.rawbackend.chainedfast');
      $args = $definition->getArguments();
      // Add the missins optional arguments
      if (!isset($args[1])) {
        $args[] = 'cache.rawbackend.couchbase';
        $definition->setArguments($args);
      }
    }

    // Override tag checksum invalidator
    // Do not replace this, because the CacheCacheTagInvalidator
    // from supercache is much better, and still uses couchbase in behind.
    //$definition = $container->getDefinition('cache_tags.invalidator.checksum');
    //if ($definition->getClass() == DatabaseCacheTagsChecksum::class) {
    //  $couchbase_definition = $container->getDefinition('cache_tags.invalidator.checksum.couchbase');
    //  $definition->setClass($couchbase_definition->getClass());
    //  $definition->setArguments($couchbase_definition->getArguments());
    //}

    // Override locking backend.
    $definition = $container->getDefinition('lock');
    if ($definition->getClass() == DatabaseLockBackend::class) {
      $couchbase_definition = $container->getDefinition('lock.couchbase');
      $definition->setClass($couchbase_definition->getClass());
      $definition->setArguments($couchbase_definition->getArguments());
    }

    // Override persistent locking backend.
    $definition = $container->getDefinition('lock.persistent');
    if ($definition->getClass() == PersistentDatabaseLockBackend::class) {
      $couchbase_definition = $container->getDefinition('lock.persistent.couchbase');
      $definition->setClass($couchbase_definition->getClass());
      $definition->setArguments($couchbase_definition->getArguments());
    }

    // Override the flood service.
    $definition = $container->getDefinition('flood');
    if ($definition->getClass() == \Drupal\Core\Flood\DatabaseBackend::class) {
      $couchbase_definition = $container->getDefinition('flood.couchbase');
      $definition->setClass($couchbase_definition->getClass());
      $definition->setArguments($couchbase_definition->getArguments());
    }

    static::deployIndexes($container->get('couchbasedrupal.manager'));
  }

  /**
   * Deploy indexes neede by the diferent components.
   * 
   * @param CouchbaseManager $manager 
   */
  protected function deployIndexes(\Drupal\couchbasedrupal\CouchbaseManager $manager) {

    foreach ($manager->listBuckets() as $name) {
      $bucket = $manager->getBucket($name);

      try {
        $bucket->queryN1QL(\CouchbaseN1qlQuery::fromString("CREATE PRIMARY INDEX ON {$name} USING GSI", NULL, TRUE));
      }
      catch (\Exception $e) {
      }

      try {
        $bucket->queryN1QL(\CouchbaseN1qlQuery::fromString("CREATE INDEX id_ix on {$name}(meta().id);"), NULL, TRUE);
      }
      catch (\Exception $e) {
      }

      // Now an index for the flood fields
      try {
        $bucket->queryN1QL(\CouchbaseN1qlQuery::fromString("CREATE INDEX id_flood_{$manager->getSitePrefix()} on {$name}(event, identifier, timestamp) WHERE META({$name}).id LIKE \"{$manager->getSitePrefix()}_flood_%\""), NULL, TRUE);
      }
      catch (\Exception $e) {
      }
    }



  }
}
